@@ -11,18 +11,17 @@ from tqdm import tqdm
 
 from thingsvision.model import build_model
 
-#from thingsvision.simple_tokenizer import SimpleTokenizer as _Tokenizer
-
-__all__ = ["available_models", "load"] #, "tokenize"]
-#_tokenizer = _Tokenizer()
+__all__ = ["available_models", "load"]
 
 _MODELS = {
     "RN50": "https://openaipublic.azureedge.net/clip/models/afeb0e10f9e5a86da6080e35cf09123aca3b358a0c3e3b6c78a7b63bc04b6762/RN50.pt",
     "ViT-B/32": "https://openaipublic.azureedge.net/clip/models/40d365715913c9da98579312b702a82c18be219cc2a73407c4526f58eba950af/ViT-B-32.pt",
 }
 
-
-def _download(url: str, root: str = os.path.expanduser("~/.cache/clip")):
+def _download(
+                url: str,
+                root: str = os.path.expanduser("~/.cache/clip"),
+):
     os.makedirs(root, exist_ok=True)
     filename = os.path.basename(url)
 
@@ -54,11 +53,17 @@ def _download(url: str, root: str = os.path.expanduser("~/.cache/clip")):
     return download_target
 
 
-def available_models():
+def available_models() -> list:
     return list(_MODELS.keys())
 
 
-def load(name: str, device: Union[str, torch.device] = "cuda" if torch.cuda.is_available() else "cpu", jit=True, pretrained=True, model_path=None):
+def load(
+        name: str,
+        device: Union[str, torch.device] = "cuda" if torch.cuda.is_available() else "cpu",
+        jit=True,
+        pretrained=True,
+        model_path=None,
+):
     if name not in _MODELS:
         raise RuntimeError(f"Model {name} not found; available models = {available_models()}")
 
@@ -123,22 +128,3 @@ def load(name: str, device: Union[str, torch.device] = "cuda" if torch.cuda.is_a
         model.float()
 
     return model, transform
-
-#NOTE: uncomment, if you want to extend THINGSvision to multi-modal feature extraction (i.e., text and featres)
-"""
-def tokenize(texts: Union[str, List[str]], context_length: int = 77):
-    if isinstance(texts, str):
-        texts = [texts]
-
-    sot_token = _tokenizer.encoder["<|startoftext|>"]
-    eot_token = _tokenizer.encoder["<|endoftext|>"]
-    all_tokens = [[sot_token] + _tokenizer.encode(text) + [eot_token] for text in texts]
-    result = torch.zeros(len(all_tokens), context_length, dtype=torch.long)
-
-    for i, tokens in enumerate(all_tokens):
-        if len(tokens) > context_length:
-            raise RuntimeError(f"Input {texts[i]} is too long for context length {context_length}")
-        result[i, :len(tokens)] = torch.tensor(tokens)
-
-    return result
-"""
