@@ -1,44 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__all__ = [
-            'load_dl',
-            'load_model',
-            'show_model',
-            'get_module_names',
-            'center_features',
-            'normalize_features',
-            'compress_features',
-            'enumerate_layers',
-            'extract_features',
-            'get_class_probabilities',
-            'get_cls_mapping_imagenet',
-            'get_digits',
-            'get_model',
-            'get_shape',
-            'json2dict',
-            'merge_features',
-            'parse_imagenet_classes',
-            'parse_imagenet_synsets',
-            'store_features',
-            'split_features',
-            'slices2tensor',
-            'tensor2slices',
-            'load_item_names',
-            'save_features',
-            'save_targets',
-            'correlation_matrix',
-            'cosine_matrix',
-            'compute_rdm',
-            'correlate_rdms',
-            'extract_features_across_models_and_datasets',
-            'extract_features_across_models_datasets_and_modules,'
-            'compare_models',
-            'get_features',
-            'compare_models_to_humans',
-            'bootstrap_',
-            ]
-
+import h5py
 import json
 import os
 import random
@@ -70,6 +33,7 @@ from torch.utils.data import DataLoader, Subset
 from torchvision import transforms as T
 from typing import Tuple, List, Iterator, Dict, Any
 
+FILE_FORMATS = ['hdf5', 'npy', 'txt', 'mat']
 
 def load_dl(
              root: str,
@@ -82,7 +46,43 @@ def load_dl(
              add_ref_imgs: bool=None,
              file_names: List[str]=None,
              transforms=None,
-             ) -> Iterator:
+) -> Iterator:
+    """Create a data loader for custom image dataset
+
+    Parameters
+    ----------
+    root : str
+        Root directory. Directory where images are stored.
+    out_path : str
+        PATH where order of images features should be stored.
+    batch_size : int (optional)
+        Number of samples (i.e., images) per mini-batch.
+    imagenet_train : bool (optional)
+        Whether ImageNet train set is used.
+    imagenet_val : bool (optional)
+        Whether ImageNet validation set is used.
+    things : bool (optional)
+        Whether THINGS database is used.
+    things_behavior : bool (optional)
+        Whether THINGS images used in behavioral experiments
+        are used.
+    add_ref_imgs : bool (optional)
+        Whether the union of the THINGS database and those
+        images that were used in behavioral experiments is used.
+    file_names : List[str] (optional)
+        List of file names. A list of file names that determines
+        the order in which image features are extracted can optionally
+        be passed.
+    transforms : Any
+        Composition of image transformations. Images must be transformed
+        into the right format for a PyTorch model.
+
+    Returns
+    -------
+    output : Iterator
+        Returns an iterator of image mini-batches.
+        Each mini-batch consists of <batch_size> samples.
+    """
     print(f'\n...Loading dataset into memory.')
     if not os.path.exists(out_path):
         os.makedirs(out_path)
@@ -108,9 +108,35 @@ def load_model(
                 pretrained: bool,
                 device: str,
                 model_path: str=None,
-                )
--> Tuple[Any, Any]:
-    """load a pretrained *torchvision* or CLIP model into memory"""
+) -> Tuple[Any, Any]:
+    """Load a pretrained *torchvision* or CLIP model into memory.
+
+    Parameters
+    ----------
+    Model_name : str
+        Model name. Name of model for which features should
+        subsequently be extracted.
+    pretrained : bool
+        Whether to load a model with pretrained or
+        randomly initialized weights into memory.
+    device : str
+        Device. Whether model weights should be moved
+        to CUDA or left on the CPU.
+    model_path : str (optional)
+        path/to/weights. If pretrained is set to False,
+        model weights can be loaded from a path on the
+        user's machine. This is useful when operating
+        on a server without network access, or when
+        features should be extracted for a model that
+        was fine-tuned (or trained) on a custom image
+        dataset.
+
+    Returns
+    -------
+    output : Tuple[model, transforms]
+        Returns the (pretrained or randomly initialized)
+        model and the corresponding image transformations.
+    """
     if re.search(r'^clip', model_name):
         if re.search(r'ViT$', model_name):
             model, transforms = clip.load(
@@ -150,6 +176,7 @@ def load_model(
 
 
 def show_model(model, model_name: str) -> str:
+    """Show architecture of model to select a layer."""
     if re.search(r'^clip', model_name):
         for l, (n, p) in enumerate(model.named_modules()):
             if l > 1:
@@ -165,7 +192,7 @@ def show_model(model, model_name: str) -> str:
 
 
 def get_module_names(model, module: str) -> list:
-    """helper to extract correct module names, if users wants to iterate over multiple modules"""
+    """Extract correct module names, iterating over multiple modules is desired."""
     module_names, _ = zip(*model.named_modules())
     return list(filter(lambda n: re.search(f'{module}$', n), module_names))
 
@@ -215,7 +242,7 @@ def extract_features_across_models_datasets_and_modules(
 
 
 def get_activation(name):
-    """store hidden unit activations at each layer of model"""
+    """Store hidden unit activations at each layer of model."""
     def hook(model, input, output):
         try:
             activations[name] = output.detach()
@@ -225,14 +252,14 @@ def get_activation(name):
 
 
 def register_hook(model):
-    """register a forward hook to store activations"""
+    """Register a forward hook to store activations."""
     for n, m in model.named_modules():
         m.register_forward_hook(get_activation(n))
     return model
 
 
 def center_features(X: np.ndarray) -> np.ndarray:
-    """center features to have zero mean"""
+    """Center features to have zero mean."""
     try:
         X -= X.mean(axis=0)
         return X
@@ -241,7 +268,7 @@ def center_features(X: np.ndarray) -> np.ndarray:
 
 
 def normalize_features(X: np.ndarray) -> np.ndarray:
-    """normalize feature vectors by their l2-norm"""
+    """Normalize feature vectors by their l2-norm."""
     try:
         X /= np.linalg.norm(X, axis=1)[:, np.newaxis]
         return X
@@ -267,7 +294,7 @@ def ensemble_featmaps(
                         alpha: float=3.,
                         beta: float=5.,
 ) -> torch.Tensor:
-    """concatenate globally (max or average) pooled feature maps"""
+    """Concatenate globally (max or average) pooled feature maps."""
     acts = [activations[''.join(('features.', str(l)))] for l in layers]
     func = torch.max if pooling == 'max' else torch.mean
     pooled_acts = [torch.tensor([list(map(func, featmaps)) for featmaps in acts_i]) for acts_i in acts]
@@ -278,6 +305,7 @@ def ensemble_featmaps(
 
 
 def compress_features(X: np.ndarray, rnd_seed: int, retained_var: float=.9) -> np.ndarray:
+    """Compress feature matrix with Principal Components Analysis (PCA)."""
     from sklearn.decomposition import PCA
     assert isinstance(rnd_seed, int), '\nTo reproduce results, random state for PCA must be defined.\n'
     pca = PCA(n_components=retained_var, svd_solver='full', random_state=rnd_seed)
@@ -287,16 +315,52 @@ def compress_features(X: np.ndarray, rnd_seed: int, retained_var: float=.9) -> n
 
 def extract_features(
                     model: Any,
-                    data_loader: Any,
+                    data_loader: Iterator,
                     module_name: str,
                     batch_size: int,
                     flatten_acts: bool,
                     device: str,
                     clip: bool=False,
                     return_probabilities: bool=False,
-                    feature_extractor=None,
 ) -> Tuple[np.ndarray, np.ndarray]:
-    """extract hidden unit activations (at specified layer) for every image in the database"""
+    """Extract hidden unit activations (at specified layer) for every image in the database.
+
+    Parameters
+    ----------
+    model : Any
+        Neural network model. The previously loaded
+        neural network model for which image features
+        should be extraced in a batch-wise manner.
+    data_loader : Iterator
+        Mini-batches. Iterator with equally sized
+        mini-batches, where each element is a
+        subsample of the full image dataset.
+    module_name : str
+        Layer name. Name of neural network layer for
+        which features should be extraced.
+    flatten_acts : bool
+        Whether activation tensor (e.g., activations
+        from an early layer of the neural network model)
+        should be transformed into a vector.
+    device : str
+        Device. Whether feature extraction should
+        be executed on CUDA or CPU.
+    clip : bool (optional)
+        Whether neural network model is a CNN-based
+        torchvision or CLIP-based model. Since CLIP
+        has a different training objective, feature
+        extraction must be performed differently.
+    return_probabilities : bool (optional)
+        Whether class probabilities (softmax predictions)
+        should be returned in addition to the feature matrix
+        and the target vector.
+
+    Returns
+    -------
+    output : Tuple[np.ndarray, np.ndarray] OR Tuple[np.ndarray, np.ndarray, np.ndarray]
+        Returns the feature matrix and the target vector OR in addition to the feature
+        matrix and the target vector, the class probabilities.
+    """
     if re.search(r'ensemble$', module_name):
         ensembles = ['conv_ensemble', 'maxpool_ensemble']
         assert module_name in ensembles, f'\nIf aggregating filters across layers and subsequently concatenating features, module name must be one of {ensembles}\n'
@@ -372,13 +436,15 @@ def store_features(
                     features: np.ndarray,
                     file_format: str,
 ) -> None:
+    """Save feature matrix to disk in pre-defined file format."""
     if not os.path.exists(PATH):
         print(f'...Output directory did not exist. Creating directories to save features.')
         os.makedirs(PATH)
-    if re.search(r'npy', file_format):
+
+    if file_format == 'npy':
         with open(pjoin(PATH, 'features.npy'), 'wb') as f:
             np.save(f, features)
-    elif re.search(r'mat', file_format):
+    elif file_format == 'mat':
         try:
             with open(pjoin(PATH, 'file_names.txt'), 'r') as f:
                 file_names = [rm_suffix(l.strip()) for l in f]
@@ -386,6 +452,10 @@ def store_features(
             scipy.io.savemat(pjoin(PATH, 'features.mat'), features)
         except FileNotFoundError:
             scipy.io.savemat(pjoin(PATH, 'features.mat'), {'features': features})
+    elif file_format == 'hdf5':
+        h5f = h5py.File(pjoin(PATH, 'features.h5'), 'w')
+        h5f.create_dataset('features', data=features)
+        h5f.close()
     else:
         np.savetxt(pjoin(PATH, 'features.txt'), features)
     print(f'...Features successfully saved to disk.\n')
@@ -396,6 +466,7 @@ def tensor2slices(
                     file_name: str,
                     features: np.ndarray,
 ) -> None:
+    """Slice four-dimensional tensor into two-dimensional matrices."""
     with open(pjoin(PATH, file_name), 'w') as outfile:
         outfile.write(f'# Array shape: {features.shape}\n')
         for i in range(features.shape[0]):
@@ -416,19 +487,6 @@ def get_shape(PATH: str, file: str) -> tuple:
     return shape
 
 
-def get_digits(string: str) -> int:
-    c = ""
-    nonzero = False
-    for i in string:
-        if i.isdigit():
-            if (int(i) == 0) and (not nonzero):
-                continue
-            else:
-                c += i
-                nonzero = True
-    return int(c)
-
-
 def slices2tensor(PATH: str, file: str) -> np.ndarray:
     slices = np.loadtxt(pjoin(PATH, file))
     shape = get_shape(PATH, file)
@@ -442,49 +500,59 @@ def split_features(
                     file_format: str,
                     n_splits: int,
 ) -> None:
-    if re.search(r'mat', file_format):
+    """Split feature matrix into <n_splits> subsamples to counteract MemoryErrors."""
+    if file_format == 'mat':
         try:
             with open(pjoin(PATH, 'file_names.txt'), 'r') as f:
                 file_names = [rm_suffix(l.strip()) for l in f]
         except FileNotFoundError:
             file_names = None
     splits = np.linspace(0, len(features), n_splits, dtype=int)
+
+    if file_format == 'hdf5':
+        h5f = h5py.File(pjoin(PATH, 'features.h5'), 'w')
+
     for i in range(1, len(splits)):
         feature_split = features[splits[i-1]:splits[i]]
-        if re.search(r'npy', file_format):
+        if file_format == 'npy':
             with open(pjoin(PATH, f'features_{i:02d}.npy'), 'wb') as f:
                 np.save(f, feature_split)
-        elif re.search(r'mat', file_format):
+        elif file_format == 'mat':
             if file_names:
                 file_name_split = file_names[splits[i-1]:splits[i]]
                 features = {file_name_split[i]: feature for i, feature in enumerate(feature_split)}
                 scipy.io.savemat(pjoin(PATH, f'features_{i:02d}.mat'), features)
             else:
                 scipy.io.savemat(pjoin(PATH, f'features_{i:02d}.mat'), {'features': features})
+        elif file_format == 'hdf5':
+            h5f.create_dataset(f'features_{i:02d}', data=feature_split)
         else:
             np.savetxt(pjoin(PATH, f'features_{i:02d}.txt'), feature_split)
 
+    if file_format == 'hdf5':
+        h5f.close()
 
 def merge_features(PATH: str, file_format: str) -> np.ndarray:
-    feature_splits = np.array([split for split in os.listdir(PATH) if re.search(r'^feature', split) and split.endswith(file_format) and re.search(r'[0-9]+$', split.rstrip(file_format))])
-    enumerations = np.array([int(re.sub(r'\D', '', feature)) for feature in feature_splits])
-    feature_splits = feature_splits[np.argsort(enumerations)]
-    if file_format == '.txt':
-        features = np.vstack([np.loadtxt(pjoin(PATH, feature)) for feature in feature_splits])
-    elif file_format == '.mat':
-        features = np.vstack([scipy.io.loadmat(pjoin(PATH, feature))['features'] for feature in feature_splits])
-    elif file_format == '.npy':
-        features = []
-        for feature in feature_splits:
-            with open(pjoin(PATH, feature), 'rb') as f:
-                features.append(np.load(f))
-        features = np.vstack(features)
+    if file_format == 'hdf5':
+        with h5py.File(pjoin(PATH, 'features.h5'), 'r') as f:
+            features = np.vstack([split[:] for split in f.values()])
     else:
-        raise Exception('\nCan only process .npy, .mat, or .txt files.\n')
+        feature_splits = np.array([split for split in os.listdir(PATH) if split.endswith(file_format) and re.search(r'^(?=^features)(?=.*[0-9]+$).*$', split.rstrip('.' + file_format))])
+        enumerations = np.array([int(re.sub(r'\D', '', feature)) for feature in feature_splits])
+        feature_splits = feature_splits[np.argsort(enumerations)]
+        if file_format == 'txt':
+            features = np.vstack([np.loadtxt(pjoin(PATH, feature)) for feature in feature_splits])
+        elif file_format == 'mat':
+            features = np.vstack([scipy.io.loadmat(pjoin(PATH, feature))['features'] for feature in feature_splits])
+        elif file_format == 'npy':
+            features = np.vstack([np.load(pjoin(PATH, feature)) for feature in feature_splits])
+        else:
+            raise Exception('\nCan only process hdf5, npy, mat, or txt files.\n')
     return features
 
 
-def parse_imagenet_synsets(PATH: str):
+def parse_imagenet_synsets(PATH: str) -> List[str]:
+    """Convert WN synsets into classes."""
     def parse_str(str):
         return re.sub(r'[^a-zA-Z]', '', str).rstrip('n').lower()
     imagenet_synsets = []
@@ -496,7 +564,8 @@ def parse_imagenet_synsets(PATH: str):
     return imagenet_synsets
 
 
-def parse_imagenet_classes(PATH: str):
+def parse_imagenet_classes(PATH: str) -> List[str]:
+    """Disambiguate ImageNet classes."""
     imagenet_classes = []
     with open(PATH, 'r') as f:
         for i, l in enumerate(f):
@@ -509,11 +578,12 @@ def parse_imagenet_classes(PATH: str):
 
 
 def get_class_intersection(imagenet_classes: list, things_objects: list) -> set:
+    """Return intersection of THINGS objects and ImageNet classes."""
     return set(things_objects).intersection(set(imagenet_classes))
 
 
 def get_cls_mapping_imagenet(PATH: str, save_as_json: bool=False) -> dict:
-    """store ImageNet classes in a idx2cls dictionary, and subsequently save as .json file"""
+    """Store ImageNet classes in an *index_to_class* dictionary, and subsequently save as .json file."""
     if re.search(r'synset', PATH.split('/')[-1]):
         imagenet_classes = parse_imagenet_synsets(PATH)
     else:
@@ -534,6 +604,7 @@ def get_class_probabilities(
                             top_k: int,
                             save_as_json: bool,
 ) -> Dict[str, Dict[str, float]]:
+    """Compute probabilities per ImageNet class."""
     file_names = open(pjoin(out_path, 'file_names.txt'), 'r').read().splitlines()
     idx2cls = get_cls_mapping_imagenet(cls_file)
     class_probas = {}
@@ -568,41 +639,49 @@ def save_features(
                     file_format: str,
                     n_splits: int=10,
 ) -> None:
+    """Save feature matrix in desired format to disk."""
+    assert file_format in FILE_FORMATS, f'\nFile format must be one of {FILE_FORMATS}.\n'
     if not os.path.exists(out_path):
         print(f'\nOutput directory did not exist. Creating directories to save features...\n')
         os.makedirs(out_path)
     # save hidden unit actvations to disk (either as one single file or as several splits)
-    if len(features.shape) == 2:
+    if len(features.shape) > 2 and file_format == 'txt':
+        print(f'\n...Cannot save 4-way tensor in a single {file_format} file.')
+        print(f'...Now slicing tensor to store as a two-dimensional matrix.\n')
+        tensor2slices(PATH=out_path, file_name='features.txt', features=features)
+        print(f'...Sliced tensor into separate parts, and saved resulting matrix as .txt file.\n')
+    else:
         try:
             store_features(PATH=out_path, features=features, file_format=file_format)
         except MemoryError:
             print(f'\n...Could not save features as one single file due to memory problems.')
-            print(f'...Now splitting features along row axis into several batches.')
+            print(f'...Now splitting features along row axis into several batches.\n')
             split_features(PATH=out_path, features=features, file_format=file_format, n_splits=n_splits)
             print(f'...Saved features in {n_splits:02d} different files, enumerated in ascending order.')
             print(f'If you want features to be splitted into more or fewer files, simply change number of splits parameter.\n')
-    else:
-        print(f'\n...Cannot save 4-way tensor in a single file.')
-        print(f'...Now slicing tensor to store as a matrix.')
-        tensor2slices(PATH=out_path, file_name='features.txt', features=features)
-        print(f'\n...Sliced tensor into separate parts, and saved resulting matrix as .txt file.\n')
 
 
 def save_targets(
                     targets: np.ndarray,
-                    out_path: str,
+                    PATH: str,
                     file_format: str,
 ) -> None:
-    if not os.path.exists(out_path):
+    """Save target vector to disk."""
+    if not os.path.exists(PATH):
         print(f'\nOutput directory did not exist. Creating directories to save targets...\n')
-        os.makedirs(out_path)
-    if re.search(r'npy', file_format):
-        with open(pjoin(out_path, 'targets.npy'), 'wb') as f:
+        os.makedirs(PATH)
+
+    if file_format == 'npy':
+        with open(pjoin(PATH, 'targets.npy'), 'wb') as f:
             np.save(f, targets)
-    elif re.search(r'mat', file_format):
-        scipy.io.savemat(pjoin(out_path, 'targets.mat'), {'targets': targets})
+    elif file_format == 'mat':
+        scipy.io.savemat(pjoin(PATH, 'targets.mat'), {'targets': targets})
+    elif file_format == 'hdf5':
+        h5f = h5py.File(pjoin(PATH, 'targets.h5'), 'w')
+        h5f.create_dataset('targets', data=targets)
+        h5f.close()
     else:
-        np.savetxt(pjoin(out_path, 'targets.txt'), targets)
+        np.savetxt(pjoin(PATH, 'targets.txt'), targets)
     print(f'...Targets successfully saved to disk.\n')
 
 # ########################################### #
@@ -612,7 +691,7 @@ def save_targets(
 
 @njit(parallel=True, fastmath=True)
 def squared_dists(F: np.ndarray) -> np.ndarray:
-    """computes squared l2-distances between feature representations in parallel"""
+    """Compute squared l2-distances between feature representations in parallel."""
     N = F.shape[0]
     D = np.zeros((N, N))
     for i in prange(N):
@@ -622,13 +701,13 @@ def squared_dists(F: np.ndarray) -> np.ndarray:
 
 
 def gaussian_kernel(F: np.ndarray) -> np.ndarray:
-    """compute dissimilarity matrix based on a Gaussian kernel"""
+    """Compute dissimilarity matrix based on a Gaussian kernel."""
     D = squared_dists(F)
     return np.exp(-D/np.mean(D))
 
 
 def correlation_matrix(F: np.ndarray, a_min: float=-1., a_max: float=1.) -> np.ndarray:
-    """compute dissimilarity matrix based on correlation distance on matrix-level"""
+    """Compute dissimilarity matrix based on correlation distance (on the matrix-level)."""
     F_c = F - F.mean(axis=1)[:, np.newaxis]
     cov = F_c @ F_c.T
     l2_norms = np.linalg.norm(F_c, axis=1)  # compute vector l2-norm across rows
@@ -638,7 +717,7 @@ def correlation_matrix(F: np.ndarray, a_min: float=-1., a_max: float=1.) -> np.n
 
 
 def cosine_matrix(F: np.ndarray, a_min: float=-1., a_max: float=1.) -> np.ndarray:
-    """compute dissimilarity matrix based on cosine distance on matrix-level"""
+    """Compute dissimilarity matrix based on cosine distance (on the matrix-level)."""
     num = F @ F.T
     l2_norms = np.linalg.norm(F, axis=1)    # compute vector l2-norm across rows
     denom = np.outer(l2_norms, l2_norms)
@@ -647,7 +726,22 @@ def cosine_matrix(F: np.ndarray, a_min: float=-1., a_max: float=1.) -> np.ndarra
 
 
 def compute_rdm(F: np.ndarray, method: str) -> np.ndarray:
-    """compute representational dissimilarity matrix based on some distance measure"""
+    """Compute representational dissimilarity matrix based on some distance measure.
+
+    Parameters
+    ----------
+    F : ndarray
+        Input array. Feature matrix of size n x m,
+        where n corresponds to the number of observations
+        and m is the number of latent dimensions.
+    method : str
+        Distance metric (e.g., correlation, cosine).
+
+    Returns
+    -------
+    output : ndarray
+        Returns the representational dissimilarity matrix.
+    """
     methods = ['correlation', 'cosine', 'euclidean', 'gaussian']
     assert method in methods, f'\nMethod to compute RDM must be one of {methods}.\n'
     if method == 'euclidean':
@@ -668,9 +762,22 @@ def correlate_rdms(
                     rdm_2: np.ndarray,
                     correlation: str='pearson',
                     ) -> float:
-    """RDMs are symmetric matrices,
-       which is why we only need to compare their upper (or lower)
-       triangular parts (main diagonal can be omitted)"""
+    """Correlate the upper triangular parts of two distinct RDMs.
+
+    Parameters
+    ----------
+    rdm_1 : ndarray
+        First RDM.
+    rdm_2 : ndarray
+        Second RDM.
+    correlation : str
+        Correlation coefficient (e.g., Spearman, Pearson).
+
+    Returns
+    -------
+    output : float
+        Returns the correlation coefficient of the two RDMs.
+    """
     triu_inds = np.triu_indices(len(rdm_1), k=1)
     corr_func = getattr(scipy.stats, ''.join((correlation, 'r')))
     rho = corr_func(rdm_1[triu_inds], rdm_2[triu_inds])[0]
@@ -685,6 +792,30 @@ def plot_rdm(
              colormap: str='cividis',
              show_plot: bool=False,
              ) -> None:
+    """Compute and plot representational dissimilarity matrix based on some distance measure.
+
+    Parameters
+    ----------
+    out_path : str
+        Output directory. Directory where to store plots.
+    F : ndarray
+        Input array. Feature matrix of size n x m,
+        where n corresponds to the number of observations
+        and m is the number of latent dimensions.
+    method : str
+        Distance metric (e.g., correlation, cosine).
+    format : str
+        Image format in which to store visualized RDM.
+    colormap : str
+        Colormap for visualization of RDM.
+    show_plot : bool
+        Whether to show visualization of RDM after storing it to disk.
+
+    Returns
+    -------
+    output : ndarray
+        Returns the representational dissimilarity matrix.
+    """
     rdm = compute_rdm(F, method)
     plt.figure(figsize=(10, 4), dpi=200)
     plt.imshow(rankdata(rdm).reshape(rdm.shape), cmap=getattr(plt.cm, colormap))
@@ -721,7 +852,7 @@ def bootstrap_(
                dissimilarity: str='correlation',
                correlation: str='pearson',
 ) -> Tuple[Dict[str, list], float]:
-    """random sampling with replacement (resampled dataset must be of equal size to the original, observed dataset)"""
+    """Randomly sample with replacement (resampled dataset must be of equal size to the original, observed dataset)"""
     human_correlations = defaultdict(list)
     N = features_i.shape[0]
     for _ in range(n_bootstraps):
@@ -752,6 +883,46 @@ def get_features(
                 batch_size: int,
                 flatten_acts: bool,
 ) -> Dict[str, Dict[str, np.ndarray]]:
+    """Extract features for a list of neural network models and corresponding modules.
+
+    Parameters
+    ----------
+    root : str
+        Root directory. Directory where images are stored.
+    out_path : str
+        PATH where order of images features should be stored.
+        Files are alphabetically sorted and features are
+        extracted accordingly.
+    model_names : List[str]
+        List of neural network models for which features
+        should be extracted.
+    module_names : List[str]
+        List of neural network layers for which features
+        should be extracted. Modules must correspond to
+        models. This should be thought of as zipped lists.
+    clip : List[bool]
+        List of Booleans which indicates whether the
+        corresponding model in the <model_names> list
+        is a CLIP-based model or not (i.e., True if
+        CLIP, else False)
+    pretrained : bool
+        Whether pretrained or randomly initialized models
+        should be loaded into memory.
+    batch_size : int
+        Integer value that determines the number of images
+        within a single mini-batch (i.e., subsample
+        of the data).
+    flatten_acts : bool
+        Whether activation tensor (e.g., activations
+        from an early layer of the neural network model)
+        should be transformed into a feature vector.
+
+    Returns
+    -------
+    output : Dict[str, Dict[str, np.ndarray]]
+        Returns a dictionary of feature matrices
+        corresponding to the selected models and layers.
+    """
     device = 'cuda' if torch.cuda.is_available() else 'cpu'
     model_features = defaultdict(dict)
     for i, model_name in enumerate(model_names):
@@ -828,16 +999,68 @@ def compare_models(
                     dissimilarity: str='correlation',
                     correlation: str='pearson',
 ) -> pd.DataFrame:
+    """Compare object representations of different models against each other.
+
+    Parameters
+    ----------
+    root : str
+        Root directory. Directory from where to load images.
+    out_path : str
+        Output directory. Directory where to store features
+        corresponding to each neural network model.
+    model_names : List[str]
+        List of neural network models whose object representations
+        should be compared against.
+    module_names : List[str]
+        List of neural network layers for which features
+        should be extracted. Modules must correspond to
+        models. This should be thought of as zipped lists.
+    pretrained : bool
+        Whether pretrained or randomly initialized models
+        should be loaded into memory.
+    batch_size : int
+        Integer value that determines the number of images
+        within a single mini-batch (i.e., subsample
+        of the data).
+    flatten_acts : bool
+        Whether activation tensor (e.g., activations
+        from an early layer of the neural network model)
+        should be transformed into a feature vector.
+    clip : List[bool]
+        List of Booleans which indicates whether the
+        corresponding model in the <model_names> list
+        is a CLIP-based model or not (i.e., True if
+        CLIP, else False)
+    save_features : bool
+        Whether to save model features or solely compare
+        their representations against each other
+        without saving the features to disk.
+    dissimilarity : str
+        Distance metric to be used to compute RDMs
+        corresponding to the model features.
+    correlation : str
+        Correlation coefficient (e.g., Spearman or Pearson)
+        to be used when performing RSA.
+
+    Returns
+    -------
+    output : pd.DataFrame
+        Returns a correlation matrix whose rows and columns
+        correspond to the names of the models in <model_names>.
+        The cell elements are the correlation coefficients
+        for each model combination. The dataframe can subsequently
+        be converted into a heatmap with matplotlib or seaborn.
+    """
     # extract features for each model and corresponding module
     model_features = get_features(
-                                    root,
-                                    out_path,
-                                    model_names,
-                                    module_names,
-                                    pretrained,
-                                    batch_size,
-                                    flatten_acts,
-                                    clip,
+                                    root=root,
+                                    out_path=out_path,
+                                    model_names=model_names,
+                                    module_names=module_names,
+                                    clip=clip,
+                                    pretrained=pretrained,
+                                    batch_size=batch_size,
+                                    flatten_acts=flatten_acts,
                                     )
     # save model features to disc
     if save_features:
@@ -861,6 +1084,6 @@ def compare_models(
 
 
 def pickle_file_(file: dict, out_path: str, f_name: str) -> None:
-    """pickle file"""
+    """Pickle any file."""
     with open(os.path.join(out_path, f_name + '.p'), 'wb') as f:
         pickle.dump(scores, f)
